use std::collections::HashMap;

use super::player_event::*;

#[derive(Debug, Default)]
pub struct PointsCounter {
    total_points: f64,
    last_dist: f64,
    last_best_dist: f64,
    delta_acc: f64,
    delta_total: f64,
    scores_log: HashMap<isize, f64>,
    behind: bool,
}

const INSTANT_SPEED_WEIGHT: f64 = 3.0;
const AVERAGE_SPEED_WEIGHT: f64 = 1.0;

impl PointsCounter {
    pub fn new() -> PointsCounter {
        PointsCounter::default()
    }

    pub fn handle_dist(&mut self, delta: f64, dist: f64) -> Vec<PlayerEvent> {
        if delta <= 0.0 {
            return Vec::new();
        }
        self.delta_acc += delta;
        self.delta_total += delta;
        if self.delta_acc <= 0.0 || self.delta_total <= 0.0 {
            return Vec::new();
        }
        let mut ret = Vec::new();
        let delta_best_dist = dist - self.last_best_dist;
        if delta_best_dist > 0.0 {
            let average_speed = dist / self.delta_total;
            self.total_points += AVERAGE_SPEED_WEIGHT * delta * average_speed;
            let instant_speed = delta_best_dist / self.delta_acc;
            self.total_points += INSTANT_SPEED_WEIGHT * delta_best_dist * instant_speed;

            self.delta_acc = 0.0;
            self.last_best_dist = dist;
            self.score_to_log(dist);
            if self.behind {
                ret.push(PlayerEvent::CatchUp);
                self.behind = false;
            }
        } else {
            let delta_dist = dist - self.last_dist;
            if delta_dist < 0.0 {
                self.score_from_log(dist);
            }
            if delta_best_dist < 0.0 {
                self.behind = true;
            }
        }
        self.last_dist = dist;
        ret
    }

    fn score_to_log(&mut self, dist: f64) {
        self.scores_log
            .insert(dist.round() as isize, self.total_points);
    }

    fn score_from_log(&mut self, dist: f64) {
        self.total_points = *self
            .scores_log
            .get(&(dist.round() as isize))
            .unwrap_or(&0.0);
    }

    pub fn score(&self) -> i32 {
        let tmp: f64 = self.total_points;
        tmp as i32
    }

    pub fn best_row(&self) -> isize {
        self.last_best_dist.round() as isize
    }

    pub fn best_dist(&self) -> f64 {
        self.last_best_dist
    }
}
